function [meanNa,stdNa,meanK,stdK,V,t]=VCstats(modelname,Vh,Vc,nrep,Na_max,K_max)

% VCstats calculates open channel statistics for multiple repetitions of a
% voltage step applied to a stochastic Hodgkin-Huxley model from:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [meanNa,stdNa,meanK,stdK,V,t]=VCstats(modelname,Vh,Vc,nrep,Na_max,K_max,scaling)
%
% meanNa is the mean number of open sodium channels at each time step t.
% stdNa is the standard deviation in number of open sodium channels at each
%       time step t.
% meanK is the mean number of open potassium channels at each time step t.
% stdK is the standard deviation in number of open potassium channels at each
%       time step t.
% V is the relative transmembrane potential in units of millivolt.
% t is the time in units of millisecond.
%
% modelname is the model type, which may be one of the following:-
%    - 'hhfloor': Deterministic Hodgkin-Huxley model with N_Na rounded down.
%    - 'hhnint':  Deterministic Hodgkin-Huxley model with N_Na rounded to
%                 nearest integer.
%    - 'fxfloor': Fox algorithm with N_Na rounded down.
%    - 'fxnint':  Fox algorithm with N_Na rounded to nearest integer.
%    - 'cw':      Chow & White algorithm.
% Vh is the holding (initial) potential in mV
% Vc is the clamp potential in mV (to which the membrane steps)
% nrep is the number of repetitions (trials)
% Na_max is the total number of sodium channels
% K_max is the total number of potassium channels
%
% Ian C. Bruce (ibruce@ieee.org)  2007 - 2009

%Setup the waitbar.
wb1 = waitbar(0,['Running ' modelname '...']);
set(wb1,'name','VCstats - 0%');

% Do one initial simulation to determine length of simulation

[N_Na,N_K,V,t] = stochasticHH_withK_VC(modelname,Vh,Vc,Na_max,K_max);

N_Na_ensemble = zeros(nrep,length(t));
N_Na_ensemble(1,:) = N_Na;
N_K_ensemble = zeros(nrep,length(t));
N_K_ensemble(1,:) = N_K;

% Repeat simulation nrep-1 times

for k = 2:nrep

    [N_Na,N_K,V,t] = stochasticHH_withK_VC(modelname,Vh,Vc,Na_max,K_max);
    
    N_Na_ensemble(k,:) = N_Na;
    N_K_ensemble(k,:) = N_K;
    
        %UPDATE WAIT-BAR every 1%
    if (mod(k,(round(nrep/100))) == 0)
        waitbar(k/nrep,wb1)
        set(wb1,'name',['VCstats - ' sprintf('%2.1f',k/nrep*100) '%'])
    end


end

%Close Waitbar
close(wb1)

meanNa=mean(N_Na_ensemble,1);
stdNa=std(N_Na_ensemble,0,1);
meanK=mean(N_K_ensemble,1);
stdK=std(N_K_ensemble,0,1);