function [m,gm,h,gh,t] = evalFoxsinglem(V,sim_time,Na_max)

% evalFox runs a simulation of a stochastic Hodgkin-Huxley model with a
% modified sodium channel that has only a single sodium activation m-particle 
% using the Chow & White (1996) algorithm and calculates the Fox & Lu (1994)
% noise term value that is required to track the Chow & White gating
% particle states at each time step.
%
% For more details, see:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [m,gm,h,gh,t] = evalFoxsinglem(V,sim_time,Na_max)
%
% m is the sodium activation particle variable
% gm is the sodium activation particle noise term
% h is the sodium inactivation particle variable
% gh is the sodium inactivation particle noise term
% t is the time in units of millisecond.
%
% V is the relative transmembrane potential in units of millivolt.
% sim_time is the simulation time in units of millisecond.
% Na_max is the number of sodium channels.
%
% Ian C. Bruce (ibruce@ieee.org), Faheem Dinath & Melissa T. Perri  2007

dt = 1e-3; % Euler integration step in ms [dt = 1us]
t = 0:dt:sim_time; % time vector in ms
max_k = length(t); % number of Euler integration steps

% Create output variables
m = zeros(1,max_k+1);  % m-particle vector
gm = zeros(1,max_k); % m-particle noise-term vector
h = zeros(1,max_k+1);  % h-particle vector
gh = zeros(1,max_k); % h-particle noise-term vector

% Calculate gating particle transition rates
markovrates = calcrates(V);

% Calculate initial modified sodium channel states
Nmh = initcwsinglem(markovrates,Na_max,dt);

% Calculate initial equivalent gating particle values
m(1) = (Nmh(2) + Nmh(4))/Na_max;
h(1) = (Nmh(3) + Nmh(4))/Na_max;

%Setup the waitbar.
% wb1 = waitbar(0,'Running...');
% set(wb1,'name','evalFoxsinglem - 0%');

for k = 1:max_k

    % Calculate modified sodium channel states for next Euler time step
    Nmh = cwsinglem(Nmh,markovrates,dt);   

    % Calculate next equivalent gating particle values   
    m(k+1) = (Nmh(2) + Nmh(4))/Na_max; % Eq. (29)
    h(k+1) = (Nmh(3) + Nmh(4))/Na_max; % Based on Eq. (14)
    
    % Calculate equivalent gating particle noise terms - Eq. (15)
    gm(k) = m(k+1) - m(k) - (markovrates.am * (1-m(k)) - markovrates.bm * m(k)) * dt;
    gh(k) = h(k+1) - h(k) - (markovrates.ah * (1-h(k)) - markovrates.bh * h(k)) * dt;

    %UPDATE WAIT-BAR every 1%
%     if (mod(k,(round(max_k/100))) == 0)
%         waitbar(k/max_k,wb1)
%         set(wb1,'name',['evalFoxsinglem - ' sprintf('%2.1f',k/max_k*100) '%'])
%     end
    
end

%Close Waitbar
% close(wb1)

% Truncate gating particle vectors to make them the same length as the
% noise term vectors
m = m(1:end-1);
h = h(1:end-1);

% end of main function evalFox

function markovrates=calcrates(V)
% Calculate sodium gating particle transition rates

    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));   % Eq. (18)
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41)); % Eq. (19)
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));  % Eq. (20)
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));                % Eq. (21)   
    
% end of subfunction calcrates

function Nmh=initcwsinglem(markovrates,Na_max,dt)
% Randomize initial modified sodium channel states
      
    % Calculate initial steady-state fraction of open m & h particles
    minf = markovrates.am/(markovrates.am+markovrates.bm);
    hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
    % Set up initial distribution of modified sodium channel states
    Nmh = zeros(1,4);
    Nmh(4) = round(Na_max*minf*hinf);
    Nmh(3) = round(Na_max*(1-minf)*hinf);
    Nmh(2) = round(Na_max*minf*(1-hinf));
    Nmh(1) = Na_max - sum(Nmh(2:4));
       
    settletime = 500; % Number of Euluer time steps over which to randomize initial conditions

    for lp=1:settletime
        
        % Update modified sodium channel states
        Nmh = cwsinglem(Nmh,markovrates,dt);   
        
    end

% end of subfunction initcwsinglem

function Nmh=cwsinglem(Nmh,markovrates,dt)
% Calculate modified sodium channel states after one Euler time step using
% the Chow & White algorithm

    % Calculate transition rates of "escapes"
    zeta = [markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah;...
            markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh]; % Based on Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,1,3,2,4,3,4];
    p_trans_next = [0,2,1,3,1,4,2,4,3];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = Nmh*zeta; % Based on Eq. (A2) of Mino et al. (ABME 2002)
        
        % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A4) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(3),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(4),...
            markovrates.am*Nmh(3),...
            markovrates.bm*Nmh(4)]/lambda); % Based on Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        
    end

% end of subfunction cwsinglem
