% fig6.m  Run simulations and plot results for Fig. 6 of:
%
%            Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%            approximation of ion channel gating models," Annals of Biomedical
%            Engineering 37(4):824838.
%
% Ian C. Bruce (ibruce@ieee.org)  2009

%% Set up parameters
Vs = [-4:1:72 76:4:140]; % Vector of relative transmembrane potentials in mV
sim_time = 1e3; % Simulation duration in ms
Na_maxs = [1e2 1e3 1e4];   % Vector of numbers of sodium channels
K_maxs = [33 333 3333];    % Vector of numbers of potassium channels

%Create analysis variable vectors
vlps = length(Vs);
nlps = length(Na_maxs);

mmean = zeros(nlps,vlps);
hmean = zeros(nlps,vlps);
nmean = zeros(nlps,vlps);

mstd = zeros(nlps,vlps);
hstd = zeros(nlps,vlps);
nstd = zeros(nlps,vlps);

am = zeros(nlps,vlps);
bm = zeros(nlps,vlps);
ah = zeros(nlps,vlps);
bh = zeros(nlps,vlps);
an = zeros(nlps,vlps);
bn = zeros(nlps,vlps);

mFoxstd = zeros(nlps,vlps);
hFoxstd = zeros(nlps,vlps);
nFoxstd = zeros(nlps,vlps);


%% Run simulations and analyze results

for vlp=1:vlps

    disp(['Run ' num2str(vlp) '/' num2str(vlps)])
    drawnow

    V = Vs(vlp);

    for nlp=1:nlps
        
        Na_max = Na_maxs(nlp);
        K_max  = K_maxs(nlp);

        disp(['Na_max = ' int2str(Na_max) '; K_max = ' int2str(K_max)])
        drawnow

        [m,gm,h,gh,n,gn,t] = evalFox(V,sim_time,Na_max,K_max);

        % Calculate means and stds of noise terms

        mmean(nlp,vlp) = mean(gm);
        hmean(nlp,vlp) = mean(gh);
        nmean(nlp,vlp) = mean(gn);

        mstd(nlp,vlp) = std(gm);
        hstd(nlp,vlp) = std(gh);
        nstd(nlp,vlp) = std(gn);

        % Calculate theoretical stds according to Fox

        am(nlp,vlp) = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
        bm(nlp,vlp) = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));
        ah(nlp,vlp) = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
        bh(nlp,vlp) = (22.57)/(1 + exp((56.0 - V)/12.5));

        if (V == 35)
            an(nlp,vlp) = (129/100); % limit((0.129*(V - 35))/(1 - exp((35 - V)/10)),V,35)
            bn(nlp,vlp) = (809/250); % limit((0.3236*(35 - V))/(1 - exp((V - 35)/10)),V,35)
        else
            an(nlp,vlp) = (0.129*(V - 35))/(1 - exp((35 - V)/10));
            bn(nlp,vlp) = (0.3236*(35 - V))/(1 - exp((V - 35)/10));
        end

        dt = t(2)-t(1);
       
        mFoxstd(nlp,vlp) = sqrt((2/Na_max)*(am(nlp,vlp)*bm(nlp,vlp))/(am(nlp,vlp)+bm(nlp,vlp)))*sqrt(dt);
        hFoxstd(nlp,vlp) = sqrt((2/Na_max)*(ah(nlp,vlp)*bh(nlp,vlp))/(ah(nlp,vlp)+bh(nlp,vlp)))*sqrt(dt);
        nFoxstd(nlp,vlp) = sqrt((2/K_max)*(an(nlp,vlp)*bn(nlp,vlp))/(an(nlp,vlp)+bn(nlp,vlp)))*sqrt(dt);

        save fig6_results

    end

end

%% Plot results

% Noise term statistics versus V

f1=figure;
set(f1,'PaperPosition',[2.25 1 4 8.5])
wysiwyg(f1)

FontName = 'Times';

ax1=subplot(3,1,1);
pl1=plot(Vs,mstd(1,:),'k-','linewidth',1.0);
hold on
pl2=plot(Vs,mstd(2,:),'k--','linewidth',1.0);
pl3=plot(Vs,mstd(3,:),'k:','linewidth',1.0);
xlim(Vs([1 end]))
pl4=plot(Vs,mFoxstd(1,:),'b-','linewidth',1.5);
plot(Vs,mFoxstd(2,:),'b--','linewidth',1.5)
plot(Vs,mFoxstd(3,:),'b:','linewidth',1.5)
set(ax1,'FontName',FontName)
title('Sodium activation')
ylabel('std[\Deltag_m]')
xlabel('V (mV)')
legend([pl1, pl2, pl3, pl4],['N_{Na}^{max} = ' int2str(Na_maxs(1))],['N_{Na}^{max} = ' int2str(Na_maxs(2))],['N_{Na}^{max} = ' int2str(Na_maxs(3))],'Fox stds');

ax2=subplot(3,1,2);
pl1=plot(Vs([1:8:77 78:2:end]),hstd(1,[1:8:77 78:2:end]),'k+','linewidth',1.0);
hold on
pl2=plot(Vs([1:8:77 78:2:end]),hstd(2,[1:8:77 78:2:end]),'ko','linewidth',1.0);
pl3=plot(Vs([1:8:77 78:2:end]),hstd(3,[1:8:77 78:2:end]),'k*','linewidth',1.0);
xlim(Vs([1 end]))
pl4=plot(Vs,hFoxstd(1,:),'b-','linewidth',1.5);
plot(Vs,hFoxstd(2,:),'b--','linewidth',1.5)
plot(Vs,hFoxstd(3,:),'b:','linewidth',1.5)
set(ax2,'FontName',FontName)
title('Sodium inactivation')
ylabel('std[\Deltag_h]')
xlabel('V (mV)')
legend([pl1, pl2, pl3,pl4],['N_{Na}^{max} = ' int2str(Na_maxs(1))],['N_{Na}^{max} = ' int2str(Na_maxs(2))],['N_{Na}^{max} = ' int2str(Na_maxs(3))],'Fox stds');

ax3=subplot(3,1,3);
pl1=plot(Vs,nstd(1,:),'k-','linewidth',1.0);
hold on
pl2=plot(Vs,nstd(2,:),'k--','linewidth',1.0);
pl3=plot(Vs,nstd(3,:),'k:','linewidth',1.0);
xlim(Vs([1 end]))
pl4=plot(Vs,nFoxstd(1,:),'b-','linewidth',1.5);
plot(Vs,nFoxstd(2,:),'b--','linewidth',1.5)
plot(Vs,nFoxstd(3,:),'b:','linewidth',1.5)
set(ax3,'FontName',FontName)
title('Potassium activation')
ylabel('std[\Deltag_n]')
xlabel('V (mV)')
legend([pl1, pl2, pl3,pl4],['N_{K}^{max} = ' int2str(K_maxs(1))],['N_{K}^{max} = ' int2str(K_maxs(2))],['N_{K}^{max} = ' int2str(K_maxs(3))],'Fox stds');
