% fig7.m  Run simulations and plot results for Fig. 7 of:
%
%            Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%            approximation of ion channel gating models," Annals of Biomedical
%            Engineering 37(4):824838.
%
% Ian C. Bruce (ibruce@ieee.org)  2009

%% Simulations for noise-term dynamics analysis
%  at V = 17 mV (top and middle panel of Fig. 7)

%% Set up parameters
V = 17;         % Relative transmembrane potential in mV
sim_time = 4e3; % Simulation duration in ms
Na_max = 1e3;   % Number of sodium channels

%% Run simulation
[m,gm,h,gh,t] = evalFoxsinglem(V,sim_time,Na_max);

%% Analyze results and plot

dt = t(2)-t(1);

% Calculate amplitude histograms for noise term
mmax = max(abs(gm));

medges = linspace(-1.1*mmax,1.1*mmax,50);

mbinsize = medges(2)-medges(1);

mamps = histc(gm,medges);

markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));

mstd = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm))*sqrt(dt);

mpoints = linspace(-1.1*mmax,1.1*mmax,101);

mstepsize = mpoints(2)-mpoints(1);

mdist = gaussian(mpoints,0,mstd);

% Calculate autocorrelation functions for noise term
[Rm,mlag] = xcorr(gm,40,'coeff');

f1=figure;
set(f1,'PaperPosition',[2.25 1 4 8.5])

wysiwyg(f1)

FontName = 'Times';

% Amplitude histogram plot

ax1=subplot(3,1,1);
h1a=fill(mpoints,mdist/sum(mdist*mstepsize)*sum(mamps*mbinsize),'b');
hold on
h1b=bar(medges,mamps,'histc');
set(ax1,'FontName',FontName)
set(h1a,'FaceColor','b','EdgeColor','none')
set(h1b,'FaceColor','none','EdgeColor','k','linewidth',1.5)
ylabel('# of occurrences')
xlabel('\Deltag_m')

% Autocorrelation function plot

ax2=subplot(3,1,2);
plot(mlag,Rm,'k','linewidth',1.5)
ylim([-0.2 1.1])
set(ax2,'FontName',FontName)
ylabel('R_m')
xlabel('Lag (\mus)')

%% Simulations for noise-term statistics vs membrane potential analysis
%  (bottom panel of Fig. 7)

%% Set up parameters
Vs = [-4:1:72 76:4:140]; % Vector of relative transmembrane potentials in mV
sim_time = 1e3; % Simulation duration in ms
Na_max = 1e3;   % Number of sodium channels

%Create analysis variable vectors
lps = length(Vs);

mmean = zeros(1,lps);
hmean = zeros(1,lps);

mstd = zeros(1,lps);
hstd = zeros(1,lps);

am = zeros(1,lps);
bm = zeros(1,lps);
ah = zeros(1,lps);
bh = zeros(1,lps);

mFoxstd = zeros(1,lps);
hFoxstd = zeros(1,lps);


%% Run simulations and analyze results

for lp=1:lps

    disp(['Run ' num2str(lp) '/' num2str(lps)])
    drawnow

    V = Vs(lp);
    [m,gm,h,gh,t] = evalFoxsinglem(V,sim_time,Na_max);

    % Calculate means and stds of noise terms

    dt = t(2)-t(1);
    
    bins = length(gm);
    
    mmean(lp) = mean(gm);
    hmean(lp) = mean(gh);
       
    mstd(lp) = std(gm);
    hstd(lp) = std(gh);

    % Calculate theoretical stds according to Fox

    am(lp) = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
    bm(lp) = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));
    ah(lp) = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
    bh(lp) = (22.57)/(1 + exp((56.0 - V)/12.5));

    mFoxstd(lp) = sqrt((2/Na_max)*(am(lp)*bm(lp))/(am(lp)+bm(lp)))*sqrt(dt);
    hFoxstd(lp) = sqrt((2/Na_max)*(ah(lp)*bh(lp))/(ah(lp)+bh(lp)))*sqrt(dt);

end

%% Plot results

ax3=subplot(3,1,3);
plot(Vs(1:4:end),mstd(1:4:end),'ko','linewidth',1.0)
xlim(Vs([1 end]))
hold on
grid on
plot(Vs,mmean,'k--','linewidth',1.0)
plot(Vs,mFoxstd,'b','linewidth',1.5)
set(ax3,'FontName',FontName)
ylabel('std[\Deltag_m]   or    E[\Deltag_m]')
xlabel('V (mV)')
legend('std[\Deltag_m]','E[\Deltag_m]','Fox std')
