function [N_Na,N_K,V,t] = stochasticHH_withK_VC(modelname,Vh,Vc,Na_max,K_max)

% stochasticHH_withK_VC runs a voltage clamp simulation of a stochastic
% Hodgkin-Huxley model of:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [N_Na,N_K,V,t] = stochasticHH_withK_VC(modelname,Vh,Vc,Na_max,K_max)
%
% N_Na is the number of open sodium channels.
% N_K is the number of open potassium channels.
% V is the relative transmembrane potential in units of millivolt.
% t is the time in units of millisecond.
%
% modelname is the model type, which may be one of the following:-
%    - 'fx':      Fox algorithm with no rounding of N_Na.
%    - 'fxnint':  Fox algorithm with N_Na rounded to nearest integer.
%    - 'cw':      Chow & White algorithm.
% Vh is the holding (initial) potential in mV
% Vc is the clamp potential in mV (to which the membrane steps)
% Na_max is the total number of sodium channels
% K_max is the total number of potassium channels
%
% Ian C. Bruce (ibruce@ieee.org), Faheem Dinath & Melissa T. Perri  2008

% Set up the appropriate functions for the chosen model
if (strcmp(modelname,'fx'))
    calcN_NaK = @fx;
    initN_NaK = @initfx;
    global m h n;
elseif (strcmp(modelname,'fxnint'))
    calcN_NaK = @fxnint;
    initN_NaK = @initfxnint;
    global m h n;
elseif (strcmp(modelname,'cw'))
    calcN_NaK = @cw;
    initN_NaK = @initcw;
    global Nmh Nn;
else
    error(['Model type "' modelname '" unknown'])
end

dt = 1e-3; % Euler integration step in ms [dt = 1us]
sim_time = 2; % simulation time in ms

t = 0:dt:sim_time; % time vector in ms
max_k = length(t); % number of Euler integration steps

% Create output variables
V = zeros(1,max_k); % Relative transmembrane potential vector
N_Na = zeros(1,max_k); % Open sodium channel count vector
N_K = zeros(1,max_k); % Open sodium channel count vector

V(1:round(0.1/dt-1)) = Vh;       % set relative membrane potential at Vh for t<0.1 ms
V(round(0.1/dt-1):end) = Vc;     % step relative membrane potential to Vc at t=0.1 ms

% Calculate gating particle transition rates for initial membrane potential
markovrates = calcrates(V(1));

% Calculate initial number of open sodium channels
[N_Na(1), N_K(1)] = initN_NaK(markovrates,Na_max,K_max,dt);

for k = 1:(max_k-1)

    % Calculate sodium gating particle transition rates for present
    % membrane potential
    markovrates = calcrates(V(k));
    
    % Calculate number of open sodium channels for next Euler time step
    [N_Na(k+1), N_K(k+1)] = calcN_NaK(markovrates,Na_max,K_max,dt);
        
end

% end of main function stochasticHH

function markovrates=calcrates(V)
% Calculate gating particle transition rates

    % Calculate sodium gating particle transition rates
    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));   % Eq. (18)
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41)); % Eq. (19)
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));  % Eq. (20)
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));                % Eq. (21)

    % Calculate potassium gating particle transition rates
    if (V == 35)
        markovrates.an = (129/100); % limit((0.129*(V - 35))/(1 - exp((35 - V)/10)),V,35)
        markovrates.bn = (809/250); % limit((0.3236*(35 - V))/(1 - exp((V - 35)/10)),V,35)
    else
        markovrates.an = (0.129*(V - 35))/(1 - exp((35 - V)/10));  % Eq. (22)
        markovrates.bn = (0.3236*(35 - V))/(1 - exp((V - 35)/10)); % Eq. (23)
    end
    
% end of subfunction calcrates

function [N_Na,N_K]=initfx(markovrates,Na_max,K_max,dt)
% Randomize initial sodium channel gating particle values according to Fox
% & Lu algorithm

    global m h n;

    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);
    n = markovrates.an/(markovrates.an+markovrates.bn);

    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        [N_Na,N_K]=fx(markovrates,Na_max,K_max,dt);
        
    end
    
% end of subfunction initfxfloor

function [N_Na,N_K]=fx(markovrates,Na_max,K_max,dt)
% Calculate number of open sodium channels after one Euler time step
% according to Fox & Lu algorithm

    global m h n;

    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    n_std = sqrt((2/K_max)*(markovrates.an*markovrates.bn)/(markovrates.an+markovrates.bn));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt);
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt);
    end
    h = hnew;

    nnew = -1;
    while (nnew<0)||(nnew>1)
        nnew = n + ((markovrates.an*(1-n))-(markovrates.bn*n))*dt + n_std*randn(1)*sqrt(dt);
    end
    n = nnew;
    
    N_Na = Na_max*m^3*h;
    N_K = K_max*n^4;

% end of subfunction fxfloor

function [N_Na,N_K]=initfxnint(markovrates,Na_max,K_max,dt)
% Randomize initial sodium channel gating particle values according to Fox
% & Lu algorithm with nint(N_Na)

    global m h n;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);
    n = markovrates.an/(markovrates.an+markovrates.bn);
   
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        [N_Na,N_K]=fxnint(markovrates,Na_max,K_max,dt);
        
    end
    
% end of subfunction initfxnint

function [N_Na,N_K]=fxnint(markovrates,Na_max,K_max,dt)
% Calculate number of open sodium channels with nint(N_Na) after one Euler
% time step according to Fox & Lu algorithm

    global m h n;

    % Calculate noise term standard deviations according to Eq. (8)
    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    n_std = sqrt((2/K_max)*(markovrates.an*markovrates.bn)/(markovrates.an+markovrates.bn));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    h = hnew;

    nnew = -1;
    while (nnew<0)||(nnew>1)
        nnew = n + ((markovrates.an*(1-n))-(markovrates.bn*n))*dt + n_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    n = nnew;
    
    N_Na = round(Na_max*m^3*h); % Eq. (9)
    N_K = round(K_max*n^4);     % Eq. (10)

% end of subfunction fxnint

function [N_Na,N_K]=initcw(markovrates,Na_max,K_max,dt)
% Randomize initial channel states according to Chow & White algorithm

    global Nmh Nn;  
   
    % Set initial number of channels in each state

    if Na_max == 1e3 % Mean steady-state values known for Na_max = 1000
        Nmh = [246     6     0     0   731    17     0     0];
        settletime = 100; % Number of Euluer time steps over which to randomize initial conditions
    else % otherwise determine approximately appropriate initial values
        % Calculate initial steady-state fraction of open h particles
        hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
        % Set up initial distribution of sodium channel states  
        Nmh = zeros(1,8);
        Nmh(5) = round(Na_max*hinf);
        Nmh(1) = Na_max - Nmh(5);
        settletime = 400; % Number of Euluer time steps over which to randomize initial conditions
    end

    % Calculate initial steady-state fraction of open n particles
    ninf = markovrates.an/(markovrates.an+markovrates.bn);
    % Set up initial distribution of potassium channel states  
    Nn = zeros(1,5);
    Nn(5) = round(K_max*ninf^4);
    Nn(1) = K_max - Nn(5);
    
    % Randomize initial conditions
    for lp=1:settletime
        [N_Na,N_K]=cw(markovrates,Na_max,K_max,dt);
    end
    
% end of subfunction initcw

function [N_Na,N_K]=cw(markovrates,Na_max,K_max,dt)
% Calculate channel states after one Euler time step using the Chow & White
% algorithm

    global Nmh Nn;
    
    % Calculate transition rates of "escapes"
    zeta = [3*markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah + 2*markovrates.am;...
            2*markovrates.bm + markovrates.ah + markovrates.am;...
            3*markovrates.bm + markovrates.ah;...
            3*markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh + 2*markovrates.am;...
            2*markovrates.bm + markovrates.bh + markovrates.am;...
            3*markovrates.bm + markovrates.bh;...
            4*markovrates.an;...
            3*markovrates.an + markovrates.bn;...
            2*markovrates.an + 2*markovrates.bn;...
            markovrates.an + 3*markovrates.bn;...
            4*markovrates.bn]; % Based on Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,2,3,3,4,1,5,2,6,3,7,4,8,5,6,6,7,7,8,1,2,2,3,3,4,4,5];
    p_trans_next = [0,2,1,3,2,4,3,5,1,6,2,7,3,8,4,6,5,7,6,8,7,2,1,3,2,4,3,5,4];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = [Nmh Nn]*zeta; % Based on Eq. (A2) of Mino et al. (ABME 2002)
    
         % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A4) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            3*markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            2*markovrates.am*Nmh(2),...
            2*markovrates.bm*Nmh(3),...
            markovrates.am*Nmh(3),...
            3*markovrates.bm*Nmh(4),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(5),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(6),...
            markovrates.ah*Nmh(3),...
            markovrates.bh*Nmh(7),...
            markovrates.ah*Nmh(4),...
            markovrates.bh*Nmh(8),...
            3*markovrates.am*Nmh(5),...
            markovrates.bm*Nmh(6),...
            2*markovrates.am*Nmh(6),...
            2*markovrates.bm*Nmh(7),...
            markovrates.am*Nmh(7),...
            3*markovrates.bm*Nmh(8),...
            4*markovrates.an*Nn(1),...
            markovrates.bn*Nn(2),...
            3*markovrates.an*Nn(2),...
            2*markovrates.bn*Nn(3),...
            2*markovrates.an*Nn(3),...
            3*markovrates.bn*Nn(4),...
            markovrates.an*Nn(4),...
            4*markovrates.bn*Nn(5)]/lambda); % Based on Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        if (ind < 22)
            Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
            Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        else
            Nn(p_trans_curr(ind)) = Nn(p_trans_curr(ind)) - 1; % decrease number of channels in current state
            Nn(p_trans_next(ind)) = Nn(p_trans_next(ind)) + 1; % increase number of channels in next state
        end
        
    end

    N_Na = Nmh(end); % Eq. (3)
    N_K = Nn(end);   % Eq. (4)

% end of subfunction cw
